@@ -63,6 +63,7 @@
 #include <glib.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 
 #ifdef ENABLE_DESKTOP_ICONS
 #include "xfdesktop-icon-view.h"
@@ -87,7 +88,7 @@
 /* disable setting the x background for bug 7442 */
 //#define DISABLE_FOR_BUG7442
 
-struct _XfceDesktopPriv
+struct _XfceDesktopPrivate
 {
     GdkScreen *gscreen;
     WnckScreen *wnck_screen;
@@ -95,9 +96,9 @@ struct _XfceDesktopPriv
 
     XfconfChannel *channel;
     gchar *property_prefix;
-    
+
     cairo_surface_t *bg_surface;
-    
+
     gint nworkspaces;
     XfceWorkspace **workspaces;
     gint current_workspace;
@@ -114,11 +115,14 @@ struct _XfceDesktopPriv
     gboolean icons_font_size_set;
     guint icons_font_size;
     guint icons_size;
+    gboolean primary;
     gboolean icons_center_text;
     gint  style_refresh_timer;
     GtkWidget *icon_view;
     gdouble system_font_size;
 #endif
+
+    gchar *last_filename;
 };
 
 enum
@@ -134,6 +138,7 @@ enum
 #ifdef ENABLE_DESKTOP_ICONS
     PROP_ICON_STYLE,
     PROP_ICON_SIZE,
+    PROP_ICON_ON_PRIMARY,
     PROP_ICON_FONT_SIZE,
     PROP_ICON_FONT_SIZE_SET,
     PROP_ICON_CENTER_TEXT,
@@ -192,12 +197,12 @@ xfce_desktop_ensure_system_font_size(XfceDesktop *desktop)
     GtkSettings *settings;
     gchar *font_name = NULL;
     PangoFontDescription *pfd;
-    
+
     gscreen = gtk_widget_get_screen(GTK_WIDGET(desktop));
-    
+
     settings = gtk_settings_get_for_screen(gscreen);
     g_object_get(G_OBJECT(settings), "gtk-font-name", &font_name, NULL);
-    
+
     pfd = pango_font_description_from_string(font_name);
     desktop->priv->system_font_size = pango_font_description_get_size(pfd);
     /* FIXME: this seems backwards from the documentation */
@@ -206,10 +211,10 @@ xfce_desktop_ensure_system_font_size(XfceDesktop *desktop)
         desktop->priv->system_font_size /= PANGO_SCALE;
     }
     XF_DEBUG("system font size is %.05f", desktop->priv->system_font_size);
-    
+
     g_free(font_name);
     pango_font_description_free(pfd);
-    
+
     return desktop->priv->system_font_size;
 }
 
@@ -217,22 +222,22 @@ static void
 xfce_desktop_setup_icon_view(XfceDesktop *desktop)
 {
     XfdesktopIconViewManager *manager = NULL;
-    
+
     switch(desktop->priv->icons_style) {
         case XFCE_DESKTOP_ICON_STYLE_NONE:
             /* nada */
             break;
-        
+
         case XFCE_DESKTOP_ICON_STYLE_WINDOWS:
             manager = xfdesktop_window_icon_manager_new(desktop->priv->gscreen);
             break;
-        
+
 #ifdef ENABLE_FILE_ICONS
         case XFCE_DESKTOP_ICON_STYLE_FILES:
             {
                 GFile *file;
                 const gchar *desktop_path;
-                
+
                 desktop_path = g_get_user_special_dir(G_USER_DIRECTORY_DESKTOP);
                 file = g_file_new_for_path(desktop_path);
                 manager = xfdesktop_file_icon_manager_new(file, desktop->priv->channel);
@@ -240,17 +245,17 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
             }
             break;
 #endif
-        
+
         default:
             g_critical("Unusable XfceDesktopIconStyle: %d.  Unable to " \
                        "display desktop icons.",
                        desktop->priv->icons_style);
             break;
     }
-    
+
     if(manager) {
         xfce_desktop_ensure_system_font_size(desktop);
-        
+
         desktop->priv->icon_view = xfdesktop_icon_view_new(manager);
         /* If the user set a custom font size, use it. Otherwise use the system
          * font size */
@@ -268,23 +273,28 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
         gtk_widget_show(desktop->priv->icon_view);
         gtk_container_add(GTK_CONTAINER(desktop), desktop->priv->icon_view);
 
+        xfdesktop_icon_view_set_primary(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
+                                        desktop->priv->primary);
+
         if(desktop->priv->icons_style == XFCE_DESKTOP_ICON_STYLE_FILES)
             g_signal_connect(G_OBJECT(manager), "hidden-state-changed",
                              G_CALLBACK(hidden_state_changed_cb), desktop);
     }
-    
+
     gtk_widget_queue_draw(GTK_WIDGET(desktop));
 }
 #endif
 
 static void
 set_imgfile_root_property(XfceDesktop *desktop, const gchar *filename,
-        gint monitor)
+                          gint monitor)
 {
+    GdkDisplay *display;
     gchar property_name[128];
-    
-    gdk_error_trap_push();
-    
+
+    display = gdk_screen_get_display(desktop->priv->gscreen);
+    gdk_x11_display_error_trap_push(display);
+
     g_snprintf(property_name, 128, XFDESKTOP_IMAGE_FILE_FMT, monitor);
     if(filename) {
         gdk_property_change(gdk_screen_get_root_window(desktop->priv->gscreen),
@@ -296,24 +306,26 @@ set_imgfile_root_property(XfceDesktop *desktop, const gchar *filename,
         gdk_property_delete(gdk_screen_get_root_window(desktop->priv->gscreen),
                             gdk_atom_intern(property_name, FALSE));
     }
-    
-    gdk_error_trap_pop_ignored();
+
+    gdk_x11_display_error_trap_pop_ignored(display);
 }
 
 static void
 set_real_root_window_surface(GdkScreen *gscreen,
-                            cairo_surface_t *surface)
+                             cairo_surface_t *surface)
 {
 #ifndef DISABLE_FOR_BUG7442
     Window xid;
+    GdkDisplay *display;
     GdkWindow *groot;
     cairo_pattern_t *pattern;
-    
+
     groot = gdk_screen_get_root_window(gscreen);
     xid = GDK_WINDOW_XID(groot);
-    
-    gdk_error_trap_push();
-    
+
+    display = gdk_screen_get_display(gscreen);
+    gdk_x11_display_error_trap_push(display);
+
     /* set root property for transparent Eterms */
     gdk_property_change(groot,
             gdk_atom_intern("_XROOTPMAP_ID", FALSE),
@@ -321,11 +333,13 @@ set_real_root_window_surface(GdkScreen *gscreen,
             GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
     /* and set the root window's BG surface, because aterm is somewhat lame. */
     pattern = cairo_pattern_create_for_surface(surface);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     gdk_window_set_background_pattern(groot, pattern);
+G_GNUC_END_IGNORE_DEPRECATIONS
     cairo_pattern_destroy(pattern);
     /* there really should be a standard for this crap... */
 
-    gdk_error_trap_pop_ignored();
+    gdk_x11_display_error_trap_pop_ignored(display);
 #endif
 }
 
@@ -349,8 +363,7 @@ create_bg_surface(GdkScreen *gscreen, gpointer user_data)
 
     TRACE("really entering");
 
-    w = gdk_screen_get_width(gscreen);
-    h = gdk_screen_get_height(gscreen);
+    xfdesktop_get_screen_dimensions (gscreen, &w, &h);
     gtk_widget_set_size_request(GTK_WIDGET(desktop), w, h);
     gtk_window_resize(GTK_WINDOW(desktop), w, h);
 
@@ -361,18 +374,84 @@ create_bg_surface(GdkScreen *gscreen, gpointer user_data)
                                                           CAIRO_CONTENT_COLOR_ALPHA, w, h);
 
     pattern = cairo_pattern_create_for_surface(desktop->priv->bg_surface);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     gdk_window_set_background_pattern(gtk_widget_get_window(GTK_WIDGET(desktop)), pattern);
+G_GNUC_END_IGNORE_DEPRECATIONS
     cairo_pattern_destroy(pattern);
 
     return desktop->priv->bg_surface;
 }
 
+static void
+set_accountsservice_user_bg(const gchar *background)
+{
+    GDBusConnection *bus;
+    GVariant *variant;
+    GError *error = NULL;
+    gchar *object_path = NULL;
+
+    bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
+    if (bus == NULL) {
+        g_warning ("Failed to get system bus: %s", error->message);
+        g_error_free (error);
+        return;
+    }
+
+    variant = g_dbus_connection_call_sync (bus,
+                                           "org.freedesktop.Accounts",
+                                           "/org/freedesktop/Accounts",
+                                           "org.freedesktop.Accounts",
+                                           "FindUserByName",
+                                           g_variant_new ("(s)", g_get_user_name ()),
+                                           G_VARIANT_TYPE ("(o)"),
+                                           G_DBUS_CALL_FLAGS_NONE,
+                                           -1,
+                                           NULL,
+                                           &error);
+
+    if (variant == NULL) {
+        DBG("Could not contact accounts service to look up '%s': %s",
+            g_get_user_name (), error->message);
+        g_error_free(error);
+        g_object_unref (bus);
+        return;
+    }
+
+    g_variant_get(variant, "(o)", &object_path);
+    g_variant_unref(variant);
+
+    variant = g_dbus_connection_call_sync (bus,
+                                           "org.freedesktop.Accounts",
+                                           object_path,
+                                           "org.freedesktop.DBus.Properties",
+                                           "Set",
+                                           g_variant_new ("(ssv)",
+                                                          "org.freedesktop.DisplayManager.AccountsService",
+                                                          "BackgroundFile",
+                                                          g_variant_new_string (background ? background : "")),
+                                           G_VARIANT_TYPE ("()"),
+                                           G_DBUS_CALL_FLAGS_NONE,
+                                           -1,
+                                           NULL,
+                                           &error);
+    if (variant != NULL) {
+        g_variant_unref (variant);
+    } else {
+        g_warning ("Failed to set the background '%s': %s", background, error->message);
+        g_clear_error (&error);
+    }
+
+    g_object_unref (bus);
+}
+
 static void
 backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     cairo_surface_t *surface = desktop->priv->bg_surface;
     GdkScreen *gscreen = desktop->priv->gscreen;
+    GdkDisplay *display;
+    gchar *new_filename = NULL;
     GdkRectangle rect;
     cairo_region_t *clip_region = NULL;
     gint i, monitor = -1, current_workspace;
@@ -381,7 +460,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 #endif
 
     TRACE("entering");
-    
+
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
     if(!XFCE_IS_BACKDROP(backdrop))
@@ -392,6 +471,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 
     TRACE("really entering");
 
+    display = gdk_display_get_default();
     current_workspace = xfce_desktop_get_current_workspace(desktop);
 
     /* Find out which monitor the backdrop is on */
@@ -403,6 +483,20 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     }
     if(monitor == -1)
         return;
+    /* notify Accountsservice of the new bg (only for monitor0) */
+    if(monitor == 0)
+    {
+        if (xfce_desktop_get_current_workspace(desktop) == 0)
+        {
+            new_filename = g_strdup(xfce_backdrop_get_image_filename(backdrop));
+            if (g_strcmp0(desktop->priv->last_filename, new_filename) != 0)
+            {
+                desktop->priv->last_filename = g_strdup(new_filename);
+                set_accountsservice_user_bg(xfce_backdrop_get_image_filename(backdrop));
+            }
+            g_free(new_filename);
+        }
+    }
 
 #ifdef G_ENABLE_DEBUG
     monitor_name = gdk_screen_get_monitor_plug_name(gscreen, monitor);
@@ -417,12 +511,14 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
         /* Spanning screens */
         GdkRectangle monitor_rect;
 
-        gdk_screen_get_monitor_geometry(gscreen, 0, &rect);
+        gdk_monitor_get_geometry(gdk_display_get_monitor(display, 0),
+                                 &rect);
         /* Get the lowest x and y value for all the monitors in
          * case none of them start at 0,0 for whatever reason.
          */
         for(i = 1; i < xfce_desktop_get_n_monitors(desktop); i++) {
-            gdk_screen_get_monitor_geometry(gscreen, i, &monitor_rect);
+            gdk_monitor_get_geometry(gdk_display_get_monitor(display, i),
+                                     &monitor_rect);
 
             if(monitor_rect.x < rect.x)
                 rect.x = monitor_rect.x;
@@ -430,12 +526,12 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
                 rect.y = monitor_rect.y;
         }
 
-        rect.width = gdk_screen_get_width(gscreen);
-        rect.height = gdk_screen_get_height(gscreen);
+        xfdesktop_get_screen_dimensions (gscreen, &rect.width, &rect.height);
         XF_DEBUG("xinerama_stretch x %d, y %d, width %d, height %d",
                  rect.x, rect.y, rect.width, rect.height);
     } else {
-        gdk_screen_get_monitor_geometry(gscreen, monitor, &rect);
+        gdk_monitor_get_geometry(gdk_display_get_monitor(display, monitor),
+                                 &rect);
         XF_DEBUG("monitor x %d, y %d, width %d, height %d",
                  rect.x, rect.y, rect.width, rect.height);
     }
@@ -456,7 +552,9 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
         for(i = 0; i < monitor; i++) {
             GdkRectangle previous_monitor;
             cairo_region_t *previous_region;
-            gdk_screen_get_monitor_geometry(gscreen, i, &previous_monitor);
+
+            gdk_monitor_get_geometry(gdk_display_get_monitor(display, i),
+                                     &previous_monitor);
 
             XF_DEBUG("previous_monitor: x: %d, y: %d, w: %d, h: %d",
                      previous_monitor.x, previous_monitor.y,
@@ -736,32 +834,43 @@ screen_set_selection(XfceDesktop *desktop)
 {
     Window xwin;
     gint xscreen;
-    gchar selection_name[100];
-    Atom selection_atom, manager_atom;
-    
+    gchar selection_name[100], common_selection_name[32];
+    Atom selection_atom, common_selection_atom, manager_atom;
+
     xwin = GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(desktop)));
-    xscreen = gdk_screen_get_number(desktop->priv->gscreen);
-    
+    xscreen = gdk_x11_screen_get_screen_number(desktop->priv->gscreen);
+
     g_snprintf(selection_name, 100, XFDESKTOP_SELECTION_FMT, xscreen);
     selection_atom = XInternAtom(gdk_x11_get_default_xdisplay(), selection_name, False);
     manager_atom = XInternAtom(gdk_x11_get_default_xdisplay(), "MANAGER", False);
 
+    g_snprintf(common_selection_name, 32, "_NET_DESKTOP_MANAGER_S%d", xscreen);
+    common_selection_atom = XInternAtom(gdk_x11_get_default_xdisplay(), common_selection_name, False);
+
     /* the previous check in src/main.c occurs too early, so workaround by
      * adding this one. */
    if(XGetSelectionOwner(gdk_x11_get_default_xdisplay(), selection_atom) != None) {
-       g_warning("%s: already running, quitting.", PACKAGE);
+       g_critical("%s: already running, quitting.", PACKAGE);
        exit(0);
    }
 
+    /* Check that _NET_DESKTOP_MANAGER_S%d isn't set, as it means another
+     * desktop manager is running, e.g. nautilus */
+    if(XGetSelectionOwner (gdk_x11_get_default_xdisplay(), common_selection_atom) != None) {
+        g_critical("%s: another desktop manager is running.", PACKAGE);
+        exit(1);
+    }
+
     XSelectInput(gdk_x11_get_default_xdisplay(), xwin, PropertyChangeMask | ButtonPressMask);
     XSetSelectionOwner(gdk_x11_get_default_xdisplay(), selection_atom, xwin, GDK_CURRENT_TIME);
+    XSetSelectionOwner(gdk_x11_get_default_xdisplay(), common_selection_atom, xwin, GDK_CURRENT_TIME);
 
     /* Check to see if we managed to claim the selection. If not,
      * we treat it as if we got it then immediately lost it */
     if(XGetSelectionOwner(gdk_x11_get_default_xdisplay(), selection_atom) == xwin) {
         XClientMessageEvent xev;
         Window xroot = GDK_WINDOW_XID(gdk_screen_get_root_window(desktop->priv->gscreen));
-        
+
         xev.type = ClientMessage;
         xev.window = xroot;
         xev.message_type = manager_atom;
@@ -784,7 +893,7 @@ screen_set_selection(XfceDesktop *desktop)
 /* gobject-related functions */
 
 
-G_DEFINE_TYPE(XfceDesktop, xfce_desktop, GTK_TYPE_WINDOW)
+G_DEFINE_TYPE_WITH_PRIVATE(XfceDesktop, xfce_desktop, GTK_TYPE_WINDOW)
 
 
 static void
@@ -792,13 +901,11 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
     GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
-    
-    g_type_class_add_private(klass, sizeof(XfceDesktopPriv));
-    
+
     gobject_class->finalize = xfce_desktop_finalize;
     gobject_class->set_property = xfce_desktop_set_property;
     gobject_class->get_property = xfce_desktop_get_property;
-    
+
     widget_class->realize = xfce_desktop_realize;
     widget_class->unrealize = xfce_desktop_unrealize;
     widget_class->button_press_event = xfce_desktop_button_press_event;
@@ -807,7 +914,7 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     widget_class->delete_event = xfce_desktop_delete_event;
     widget_class->popup_menu = xfce_desktop_popup_menu;
     widget_class->style_updated = xfce_desktop_style_updated;
-    
+
     signals[SIG_POPULATE_ROOT_MENU] = g_signal_new("populate-root-menu",
                                                    XFCE_TYPE_DESKTOP,
                                                    G_SIGNAL_RUN_LAST,
@@ -817,7 +924,7 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
                                                    g_cclosure_marshal_VOID__OBJECT,
                                                    G_TYPE_NONE, 1,
                                                    GTK_TYPE_MENU_SHELL);
-    
+
     signals[SIG_POPULATE_SECONDARY_ROOT_MENU] = g_signal_new("populate-secondary-root-menu",
                                                              XFCE_TYPE_DESKTOP,
                                                              G_SIGNAL_RUN_LAST,
@@ -854,6 +961,13 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
                                                       8, 192, DEFAULT_ICON_SIZE,
                                                       XFDESKTOP_PARAM_FLAGS));
 
+    g_object_class_install_property(gobject_class, PROP_ICON_ON_PRIMARY,
+                                    g_param_spec_boolean("primary",
+                                                         "primary",
+                                                         "primary",
+                                                         FALSE,
+                                                         XFDESKTOP_PARAM_FLAGS));
+
     g_object_class_install_property(gobject_class, PROP_ICON_FONT_SIZE,
                                     g_param_spec_uint("icon-font-size",
                                                       "icon font size",
@@ -897,9 +1011,8 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
 static void
 xfce_desktop_init(XfceDesktop *desktop)
 {
-    desktop->priv = G_TYPE_INSTANCE_GET_PRIVATE(desktop, XFCE_TYPE_DESKTOP,
-                                                XfceDesktopPriv);
-    
+    desktop->priv = xfce_desktop_get_instance_private(desktop);
+
     gtk_window_set_type_hint(GTK_WINDOW(desktop), GDK_WINDOW_TYPE_HINT_DESKTOP);
     /* Accept focus is needed for the menu pop up either by the menu key on
      * the keyboard or Shift+F10. */
@@ -945,6 +1058,11 @@ xfce_desktop_set_property(GObject *object,
                                        g_value_get_uint(value));
             break;
 
+        case PROP_ICON_ON_PRIMARY:
+            xfce_desktop_set_primary(desktop,
+                                       g_value_get_boolean(value));
+            break;
+
         case PROP_ICON_FONT_SIZE:
             xfce_desktop_set_icon_font_size(desktop,
                                             g_value_get_uint(value));
@@ -995,6 +1113,10 @@ xfce_desktop_get_property(GObject *object,
             g_value_set_uint(value, desktop->priv->icons_size);
             break;
 
+        case PROP_ICON_ON_PRIMARY:
+            g_value_set_boolean(value, desktop->priv->primary);
+            break;
+
         case PROP_ICON_FONT_SIZE:
             g_value_set_uint(value, desktop->priv->icons_font_size);
             break;
@@ -1031,26 +1153,25 @@ xfce_desktop_realize(GtkWidget *widget)
     Window xid;
     GdkWindow *groot;
     WnckScreen *wnck_screen;
-    
+
     TRACE("entering");
 
     gtk_window_set_screen(GTK_WINDOW(desktop), desktop->priv->gscreen);
-    sw = gdk_screen_get_width(desktop->priv->gscreen);
-    sh = gdk_screen_get_height(desktop->priv->gscreen);
+    xfdesktop_get_screen_dimensions (desktop->priv->gscreen, &sw, &sh);
 
     g_signal_connect(G_OBJECT(desktop->priv->gscreen),
                      "monitors-changed",
                      G_CALLBACK(xfce_desktop_monitors_changed),
                      desktop);
-    
+
     /* chain up */
     GTK_WIDGET_CLASS(xfce_desktop_parent_class)->realize(widget);
-    
+
     gtk_window_set_title(GTK_WINDOW(desktop), _("Desktop"));
-    
+
     gtk_widget_set_size_request(GTK_WIDGET(desktop), sw, sh);
     gtk_window_move(GTK_WINDOW(desktop), 0, 0);
-    
+
     atom = gdk_atom_intern("_NET_WM_WINDOW_TYPE_DESKTOP", FALSE);
     gdk_property_change(gtk_widget_get_window(GTK_WIDGET(desktop)),
             gdk_atom_intern("_NET_WM_WINDOW_TYPE", FALSE),
@@ -1059,20 +1180,20 @@ xfce_desktop_realize(GtkWidget *widget)
 
     xid = GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(desktop)));
     groot = gdk_screen_get_root_window(desktop->priv->gscreen);
-    
+
     gdk_property_change(groot,
             gdk_atom_intern("XFCE_DESKTOP_WINDOW", FALSE),
             gdk_atom_intern("WINDOW", FALSE), 32,
             GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
-    
+
     gdk_property_change(groot,
             gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
             gdk_atom_intern("WINDOW", FALSE), 32,
             GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
-    
+
     screen_set_selection(desktop);
 
-    wnck_screen = wnck_screen_get(gdk_screen_get_number(desktop->priv->gscreen));
+    wnck_screen = wnck_screen_get(gdk_x11_screen_get_screen_number(desktop->priv->gscreen));
     desktop->priv->wnck_screen = wnck_screen;
 
     /* Watch for single workspace setting changes */
@@ -1098,7 +1219,7 @@ xfce_desktop_realize(GtkWidget *widget)
             G_CALLBACK(screen_composited_changed_cb), desktop);
 
     gtk_widget_add_events(GTK_WIDGET(desktop), GDK_EXPOSURE_MASK);
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     xfce_desktop_setup_icon_view(desktop);
 #endif
@@ -1110,10 +1231,11 @@ static void
 xfce_desktop_unrealize(GtkWidget *widget)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(widget);
+    GdkDisplay  *display;
     gint i;
     GdkWindow *groot;
     gchar property_name[128];
-    
+
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
     /* disconnect all the xfconf settings to this desktop */
@@ -1122,22 +1244,23 @@ xfce_desktop_unrealize(GtkWidget *widget)
     g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
                                          G_CALLBACK(xfce_desktop_monitors_changed),
                                          desktop);
-    
+
     if(gtk_widget_get_mapped(widget))
         gtk_widget_unmap(widget);
     gtk_widget_set_mapped(widget, FALSE);
-    
+
     gtk_container_forall(GTK_CONTAINER(widget),
                          (GtkCallback)gtk_widget_unrealize,
                          NULL);
-    
+
     g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
             G_CALLBACK(screen_size_changed_cb), desktop);
     g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
             G_CALLBACK(screen_composited_changed_cb), desktop);
 
-    gdk_error_trap_push();
-    
+    display = gdk_screen_get_display(desktop->priv->gscreen);
+    gdk_x11_display_error_trap_push(display);
+
     groot = gdk_screen_get_root_window(desktop->priv->gscreen);
     gdk_property_delete(groot, gdk_atom_intern("XFCE_DESKTOP_WINDOW", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE));
@@ -1145,7 +1268,9 @@ xfce_desktop_unrealize(GtkWidget *widget)
 #ifndef DISABLE_FOR_BUG7442
     gdk_property_delete(groot, gdk_atom_intern("_XROOTPMAP_ID", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("ESETROOT_PMAP_ID", FALSE));
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     gdk_window_set_background_pattern(groot, NULL);
+G_GNUC_END_IGNORE_DEPRECATIONS
 #endif
 
     if(desktop->priv->workspaces) {
@@ -1158,21 +1283,21 @@ xfce_desktop_unrealize(GtkWidget *widget)
         desktop->priv->workspaces = NULL;
     }
 
-    gdk_flush();
-    gdk_error_trap_pop_ignored();
+    gdk_display_flush(display);
+    gdk_x11_display_error_trap_pop_ignored(display);
 
     if(desktop->priv->bg_surface) {
         cairo_surface_destroy(desktop->priv->bg_surface);
         desktop->priv->bg_surface = NULL;
     }
-    
+
     gtk_window_set_icon(GTK_WINDOW(widget), NULL);
 
     g_object_unref(G_OBJECT(gtk_widget_get_window(widget)));
     gtk_widget_set_window(widget, NULL);
 
     gtk_selection_remove_all(widget);
-    
+
     gtk_widget_set_realized(widget, FALSE);
 }
 
@@ -1196,12 +1321,11 @@ xfce_desktop_button_press_event(GtkWidget *w,
                 return FALSE;
 #endif
             /* no icons on the desktop, grab the focus and pop up the menu */
-            if(!gtk_widget_has_grab(w)) {
+            if(!gtk_widget_has_grab(w))
                 gtk_grab_add(w);
 
-                xfce_desktop_popup_root_menu(desktop, button, evt->time);
-                return TRUE;
-            }
+            xfce_desktop_popup_root_menu(desktop, button, evt->time);
+            return TRUE;
         } else if(button == 2 || (button == 1 && (state & GDK_SHIFT_MASK)
                                   && (state & GDK_CONTROL_MASK)))
         {
@@ -1213,7 +1337,7 @@ xfce_desktop_button_press_event(GtkWidget *w,
             return TRUE;
         }
     }
-    
+
     return FALSE;
 }
 
@@ -1246,7 +1370,7 @@ xfce_desktop_popup_menu(GtkWidget *w)
         button = 0;
         etime = gtk_get_current_event_time();
     }
-    
+
     xfce_desktop_popup_root_menu(XFCE_DESKTOP(w), button, etime);
 
     gdk_event_free((GdkEvent*)evt);
@@ -1258,9 +1382,9 @@ xfce_desktop_draw(GtkWidget *w,
                   cairo_t *cr)
 {
     GList *children, *l;
-    
+
     /*TRACE("entering");*/
-    
+
     children = gtk_container_get_children(GTK_CONTAINER(w));
     for(l = children; l; l = l->next) {
         gtk_container_propagate_draw(GTK_CONTAINER(w),
@@ -1268,7 +1392,7 @@ xfce_desktop_draw(GtkWidget *w,
                                      cr);
     }
     g_list_free(children);
-    
+
     return FALSE;
 }
 
@@ -1278,7 +1402,7 @@ xfce_desktop_delete_event(GtkWidget *w,
 {
     if(XFCE_DESKTOP(w)->priv->session_logout_func)
         XFCE_DESKTOP(w)->priv->session_logout_func();
-    
+
     return TRUE;
 }
 
@@ -1304,8 +1428,10 @@ style_refresh_cb(gpointer *w)
 
     if(desktop->priv->bg_surface) {
         pattern = cairo_pattern_create_for_surface(desktop->priv->bg_surface);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         gdk_window_set_background_pattern(gtk_widget_get_window(GTK_WIDGET(desktop)),
                                           pattern);
+G_GNUC_END_IGNORE_DEPRECATIONS
         cairo_pattern_destroy(pattern);
     }
 
@@ -1361,6 +1487,8 @@ xfce_desktop_connect_settings(XfceDesktop *desktop)
                            G_OBJECT(desktop), "icon-style");
     xfconf_g_property_bind(channel, ICONS_PREFIX "icon-size", G_TYPE_UINT,
                            G_OBJECT(desktop), "icon-size");
+    xfconf_g_property_bind(channel, ICONS_PREFIX "primary", G_TYPE_BOOLEAN,
+                           G_OBJECT(desktop), "primary");
     xfconf_g_property_bind(channel, ICONS_PREFIX "font-size", G_TYPE_UINT,
                            G_OBJECT(desktop), "icon-font-size");
     xfconf_g_property_bind(channel, ICONS_PREFIX "use-custom-font-size",
@@ -1434,7 +1562,7 @@ xfce_desktop_new(GdkScreen *gscreen,
                  const gchar *property_prefix)
 {
     XfceDesktop *desktop;
-    
+
     g_return_val_if_fail(channel && property_prefix, NULL);
 
     desktop = g_object_new(XFCE_TYPE_DESKTOP, NULL);
@@ -1443,12 +1571,14 @@ xfce_desktop_new(GdkScreen *gscreen,
         gscreen = gdk_display_get_default_screen(gdk_display_get_default());
     gtk_window_set_screen(GTK_WINDOW(desktop), gscreen);
     desktop->priv->gscreen = gscreen;
-    
-    desktop->priv->channel = g_object_ref(G_OBJECT(channel));
+
+    desktop->priv->channel = XFCONF_CHANNEL(g_object_ref(G_OBJECT(channel)));
     desktop->priv->property_prefix = g_strdup(property_prefix);
 
     xfce_desktop_connect_settings(desktop);
-    
+
+    desktop->priv->last_filename = g_strdup("");
+
     return GTK_WIDGET(desktop);
 }
 
@@ -1456,24 +1586,8 @@ gint
 xfce_desktop_get_n_monitors(XfceDesktop *desktop)
 {
     g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), 0);
-    
-    return gdk_screen_get_n_monitors(desktop->priv->gscreen);
-}
 
-gint
-xfce_desktop_get_width(XfceDesktop *desktop)
-{
-    g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), -1);
-    
-    return gdk_screen_get_width(desktop->priv->gscreen);
-}
-
-gint
-xfce_desktop_get_height(XfceDesktop *desktop)
-{
-    g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), -1);
-    
-    return gdk_screen_get_height(desktop->priv->gscreen);
+    return gdk_display_get_n_monitors(gdk_screen_get_display(desktop->priv->gscreen));
 }
 
 void
@@ -1482,16 +1596,16 @@ xfce_desktop_set_icon_style(XfceDesktop *desktop,
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop)
                      && style <= XFCE_DESKTOP_ICON_STYLE_FILES);
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     if(style == desktop->priv->icons_style)
         return;
-    
+
     if(desktop->priv->icon_view) {
         gtk_widget_destroy(desktop->priv->icon_view);
         desktop->priv->icon_view = NULL;
     }
-    
+
     desktop->priv->icons_style = style;
     if(gtk_widget_get_realized(GTK_WIDGET(desktop)))
         xfce_desktop_setup_icon_view(desktop);
@@ -1542,7 +1656,7 @@ XfceDesktopIconStyle
 xfce_desktop_get_icon_style(XfceDesktop *desktop)
 {
     g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), XFCE_DESKTOP_ICON_STYLE_NONE);
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     return desktop->priv->icons_style;
 #else
@@ -1555,13 +1669,13 @@ xfce_desktop_set_icon_size(XfceDesktop *desktop,
                            guint icon_size)
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     if(icon_size == desktop->priv->icons_size)
         return;
-    
+
     desktop->priv->icons_size = icon_size;
-    
+
     if(desktop->priv->icon_view) {
         xfdesktop_icon_view_set_icon_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
                                           icon_size);
@@ -1569,18 +1683,35 @@ xfce_desktop_set_icon_size(XfceDesktop *desktop,
 #endif
 }
 
+void
+xfce_desktop_set_primary(XfceDesktop *desktop,
+                           gboolean primary)
+{
+    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
+
+    if(primary == desktop->priv->primary)
+        return;
+
+    desktop->priv->primary = primary;
+
+    if(desktop->priv->icon_view) {
+        xfdesktop_icon_view_set_primary(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
+                                        primary);
+    }
+}
+
 void
 xfce_desktop_set_icon_font_size(XfceDesktop *desktop,
                                 guint font_size_points)
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     if(font_size_points == desktop->priv->icons_font_size)
         return;
-    
+
     desktop->priv->icons_font_size = font_size_points;
-    
+
     if(desktop->priv->icons_font_size_set && desktop->priv->icon_view) {
         xfdesktop_icon_view_set_font_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
                                           font_size_points);
@@ -1593,13 +1724,13 @@ xfce_desktop_set_use_icon_font_size(XfceDesktop *desktop,
                                     gboolean use_icon_font_size)
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     if(use_icon_font_size == desktop->priv->icons_font_size_set)
         return;
-    
+
     desktop->priv->icons_font_size_set = use_icon_font_size;
-    
+
     if(desktop->priv->icon_view) {
         if(!use_icon_font_size) {
             xfce_desktop_ensure_system_font_size(desktop);
@@ -1618,7 +1749,7 @@ xfce_desktop_set_center_text (XfceDesktop *desktop,
                               gboolean center_text)
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
+
 #ifdef ENABLE_DESKTOP_ICONS
     if(center_text == desktop->priv->icons_center_text)
         return;
@@ -1688,7 +1819,7 @@ void
 xfce_desktop_thaw_updates(XfceDesktop *desktop)
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
+
     desktop->priv->updates_frozen = FALSE;
     if(gtk_widget_get_realized(GTK_WIDGET(desktop)))
         xfce_desktop_monitors_changed(desktop->priv->gscreen, desktop);
@@ -1710,9 +1841,9 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
     GdkScreen *screen;
     GtkWidget *menu;
     GList *menu_children;
-    
+
     DBG("entering");
-    
+
     if(gtk_widget_has_screen(GTK_WIDGET(desktop)))
         screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
     else
@@ -1808,16 +1939,6 @@ xfce_desktop_refresh(XfceDesktop *desktop, gboolean advance_wallpaper)
     /* If we're only advancing the wallpaper we can exit here */
     if(advance_wallpaper)
         return;
-
-#ifdef ENABLE_DESKTOP_ICONS
-    /* reload icon view */
-    if(desktop->priv->icon_view) {
-        gtk_widget_destroy(desktop->priv->icon_view);
-        desktop->priv->icon_view = NULL;
-    }
-    xfce_desktop_setup_icon_view(desktop);
-#endif
-
 }
 
 void xfce_desktop_arrange_icons(XfceDesktop *desktop)
